filesystem=love.filesystem
keyboard=love.keyboard
graphics=love.graphics
joystick=love.joystick
physics=love.physics
system=love.system
thread=love.thread
window=love.window
audio=love.audio
event=love.event
image=love.image
mouse=love.mouse
sound=love.sound
timer=love.timer
touch=love.touch
font=love.font
for k,v in pairs(love.math) do
	math[k]=v
end
love.math=nil

table.unpack=unpack; unpack=nil
table.pack=function(...)
	if class(...)=="table" then
		return ...
	else
		return {...}
	end
end
toboolean=function(v)
	if v==true or v=="true" or v==1 or v=="1" then
		return true
	elseif v==false or v=="false" or v==0 or v=="0" then
		return false
	end
end

function table.reverse(t,min,max)
	local c={}
	for i=max or #t,min or 1,-1 do
		c[#c+1]=t[i]
	end
	return c
end
function table.sub(t,min,max)
  local c={}
  for i=min or 1,max or #t do
    c[#c+1]=t[i]
  end
  return c
end
function table.find(t,s,min,max)
  local c={}
  for i=min or 1,max or #t do
    if type(t[i])==s then
      c[#c+1]=t[i]
    end
  end
  return c
end
function table.iterate(t,min,max,b,f,...)
  for i=min or 1,max or #t do
    if b==true then
      t[i]=f(i,...)
    else
      f(i,...)
    end
  end
end
function table.step(t,min,max,n)
  min=min or 0
  max=max or min+1
  n=n or 0
  return function(b,f,...)
    for i=min+n,max+n do
      if b==true then
        t[i]=f(i,...)
      else
        f(i,...)
      end
    end
    n=n+((max+n)-(min+n))+1
  end
end
function table.print(s,t)
	print(s.."={")
	for k,v in pairs(t) do
		if type(v)=="string" then
			v="\""..v.."\""
		end
		if type(k)=="string" then
			k="\""..k.."\""
		end
		print(" "..k.."="..tostring(v))
	end
	print("}")
end
function table.shift(t,n,s,min,max)
  local c={}
  for i=min or 1,max or #t do
    if i>n then
      c[i+s]=t[i]
    else
      c[i]=t[i]
    end
  end
  return c
end
function table.random(t,l)
  local c={}
  local n=#t
  for i=1,l do
    c[i]=t[math.random(1,n)]
  end
  return c
end

function string.index(s,i)
	return string.char(string.byte(s,i))
end
function string.random(s,l)
  local b=""
  local n=#s
  for _=1,l do
    b=b..string.index(s,math.random(1,n))
  end
  return b
end

function math.digits(n)
  return #lua.string(n)
end
function math.limit(min,max)
	return function(n)
    if n<min then
      n=min
    elseif n>max then
      n=max
    end
    return n
  end
end
function math.loop(min,max)
  return function(n)
    if n<min then
      n=max
    elseif n>max then
      n=min
    end
    return n
  end
end
function math.even(n)
	return n%2==0
end
function math.odd(n)
	return n%2~=0
end
function math.fact(n)
	for i=1,n-1 do
		n=n*i
	end
	return n
end
function math.erratic(n)
  local s=math.random(1,n)
	for _=1,math.random(1,n) do
		s=s*math.random(1,9)
	end
	return s
end
function math.intersect(x,y,w,h,xx,yy,ww,hh)
  return x+w>xx and x<xx+ww and y+h>yy and y<yy+hh
end
function math.collisionX(x,y,w,h,xx,yy,ww,hh)
  local l=(x+w)-xx
  local r=(xx+ww)-x
  local u=(y+h)-yy
  local d=(yy+hh)-y
  if math.intersect(x,y,w,h,xx,yy,ww,hh) then
    if r<l and r<u and r<d then x=xx+ww
    elseif l<r and l<u and l<d then x=xx-w
    elseif r<l and r==u and r~=d then x=xx+ww
    elseif l<r and l==u and l~=d then x=xx-w
    elseif r<l and r~=u and r==d then x=xx+ww
    elseif l<r and l~=u and l==d then x=xx-w
    end
  end
  return x
end
function math.collisionY(x,y,w,h,xx,yy,ww,hh)
  local l=(x+w)-xx
  local r=(xx+ww)-x
  local u=(y+h)-yy
  local d=(yy+hh)-y
  if math.intersect(x,y,w,h,xx,yy,ww,hh) then
    if u<d and u<l and u<r then y=yy-h
    elseif d<u and d<l and d<r then y=yy+hh
    elseif r<l and r==u and r~=d then y=yy-h
    elseif l<r and l==u and l~=d then y=yy-h
    elseif r<l and r~=u and r==d then y=yy+hh
    elseif l<r and l~=u and l==d then y=yy+hh
    end
  end
  return y
end

os.current={
  time=os.time(),
  clock=os.clock()
}
function os.interval()
  local t=0
  return function(n,m)
    t=t+n
    if t>=m then
      t=0
      return true
    else
      return false
    end
  end
end
function os.update(k)
  os.current[k]=os[k]()
end
function os.count(k)
  local t=os.current[k]
  return function(n)
    if t+n<=os[k]() then
      t=os[k]()
      return true
    else
      return false
    end
  end
end

display={}
function display.new(x,y,w,h,xx,yy,ww,hh)
  return {x=x or 0,y=y or 0,w=w or 0,h=h or 0,c={x=xx or 0,y=yy or 0,w=ww or 1,h=hh or 1}}
end
function display.translate(d,x,y)
  d.c.x=d.x-(x or 0)*d.c.w
  d.c.y=d.y-(y or 0)*d.c.h
end
function display.scale(d,w,h)
  d.c.w=w or d.c.w
  d.c.h=h or d.c.h
end
function display.apply(d)
  graphics.translate(d.c.x,d.c.y)
  graphics.scale(d.c.w,d.c.h)
end
function display.origin()
  graphics.origin()
end
function display.rectangle(d,m,...)
  local r,g,b,a=graphics.getColor()
  graphics.setColor(...)
  graphics.rectangle(m,d.x,d.y,d.w,d.h)
  graphics.setColor(r,g,b,a)
end
function display.stencil(d)
  love.graphics.setStencilTest("greater",0)
  graphics.stencil(function() graphics.rectangle("fill",d.x,d.y,d.w,d.h) end,"increment",1)
end
function display.reset()
  graphics.setStencilTest()
end


texture={}
function texture.new(i,...)
  local i=graphics.newImage(i)
  local b=graphics.newSpriteBatch(i,...)
  return {i=i,w=i:getWidth(),h=i:getHeight(),b=b}
end
function texture.create(t,i,x,y,w,h,r,g,b,a)
  t[i]={q=graphics.newQuad(x,y,w,h,t.w,t.h),x=x,y=y,w=w,h=h}
end
function texture.getImage(t)
  return t.i
end
function texture.getBatch(t)
  return t.b
end
function texture.getWidth(t,i)
  if i==nil then
    return t.w
  end
  return t[i].w
end
function texture.getHeight(t,i)
  if i==nil then
    return t.w
  end
  return t[i].w
end
function texture.getX(t,i)
  return t[i].x
end
function texture.getY(t,i)
  return t[i].y
end
function texture.getQuad(t,i)
  return t[i].q
end
function texture.visible(d,x,y,w,h)
  return math.intersect(x*d.c.w,y*d.c.h,w*d.c.w,h*d.c.h,d.x-d.c.x,d.y-d.c.y,d.w,d.h)
end
function texture.add(t,tt,i,x,y,r,w,h,...)
  for ii=1,#t do
    local d=t[ii]
    if texture.visible(d,x,y,w,h) then
      return tt.b:add(tt[i].q,x,y,r,w/tt[i].w,h/tt[i].h,...)
    end
  end
  return nil
end
function texture.set(t,tt,i,ii,x,y,r,w,h,...)
  for ii=1,#t do
    local d=t[ii]
    if texture.visible(d,x,y,w,h) then
      return tt.b:set(ii,x,y,r,w/tt[i].w,h/tt[i].h,...)
    end
  end
  return nil
end
function texture.getBufferSize(t)
  return t.b:getBufferSize()
end
function texture.getColor(t)
  return t.b:getColor()
end
function texture.getCount(t)
  return t.b:getCount()
end
function texture.getTexture(t)
  return t.b:getTexture(t)
end
function texture.setBufferSize(t)
  return t.b:setBufferSize()
end
function texture.setColor(t,...)
  return t.b:setColor(...)
end
function texture.setTexture(t,i)
  t.i=graphics.newImage(i)
  return t.b:setTexture(t.i)
end
function texture.flush(t)
  t.b:flush()
end
function texture.clear(t)
  t.b:clear()
end
function texture.draw(t,...)
  graphics.draw(t.b,...)
end
